/*
 *  Copyright (C) 2015 Jeroen Domburg <jeroen at spritesmods.com>
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <ft2build.h>
#include <cairo/cairo.h>
#include <cairo/cairo-ft.h>
#include FT_FREETYPE_H
#include "font.h"

static FT_Library ftlib;

void fontInit() {
	if (FT_Init_FreeType(&ftlib)) {
		printf("Error initializing Freetype lib\n");
		return;
	}
}

//Try to load a font. Yes, we can use caching or whatever, but I truly can't
//be arsed.
cairo_font_face_t *fontLoad(char *file, int faceidx) {
	int err;
	char buff[1024];
	FT_Face face;
	cairo_font_face_t *cairoface;
	static const cairo_user_data_key_t key;

	strcpy(buff, FONTPATH);
	strcat(buff, file);
	err=FT_New_Face(ftlib, buff, faceidx, &face);
	if (err) {
		printf("Font: Failed to create face for file %s faceidx %d\n", buff, faceidx);
		return NULL;
	}
	cairoface=cairo_ft_font_face_create_for_ft_face(face, 0);
	err=cairo_font_face_set_user_data (cairoface, &key, face, (cairo_destroy_func_t) FT_Done_Face);
	if (err) {
		cairo_font_face_destroy (cairoface);
		FT_Done_Face (face);
		return NULL;
	}
	return cairoface;
}

void fontUnload(cairo_font_face_t *font) {
	cairo_font_face_destroy(font);
}

