/*
Voom testbed
*/
/*
 *  Copyright (C) 2015 Jeroen Domburg <jeroen at spritesmods.com>
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <cairo/cairo.h>
#include <cairo/cairo-xlib.h>
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/select.h>
#include <time.h>
#include <sys/time.h>
#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdint.h>
#include "font.h"

#define SIZEX 800
#define SIZEY 600
#define FOV (M_PI/2)
#define FARPT 250
#define NEARPT 10

int mode=0;

typedef struct __attribute__((packed)) {
	int16_t x;
	int16_t y;
} DoomVertex;


typedef struct __attribute__((packed)) {
	int16_t vFrom;	//start vertex
	int16_t vTo;	//end vertex
	int16_t flags;
	int16_t types;
	int16_t tag;
	int16_t sdRight;	//right side sidedef index
	int16_t sdLeft;		//left side sidedef index
} DoomLine;

typedef struct __attribute__((packed)) {
	int16_t xOff;
	int16_t yOff;
	char texUp[8];
	char texDwn[8];
	char texMid[8];
	int16_t sector;
} DoomSide;

typedef struct __attribute__((packed)) {
	int16_t vStart;	//vertex start
	int16_t vEnd;	//vertex end
	int16_t angle;
	int16_t linedef; //linedef index
	int16_t dir;
	int16_t offset;
} DoomSeg;

typedef struct __attribute__((packed)) {
	int16_t sCount; //segment count
	int16_t sStart; //start segment index
} DoomSsec;

typedef struct __attribute__((packed)) {
	int16_t zFloor;
	int16_t zCeil;
	char fFloor[8];
	char fCeil[8];
	int16_t lightLevel;
	int16_t flags;
	int16_t tag;
} DoomSec;

typedef struct __attribute__((packed)) {
	int16_t yhi;
	int16_t ylo;
	int16_t xlo;
	int16_t xhi;
} DoomBbox;

typedef struct __attribute__((packed)) {
	DoomVertex lineStart;
	DoomVertex lineD;
	DoomBbox bbRight;
	DoomBbox bbLeft;
	int16_t rChild;
	int16_t lChild;
} DoomNode;

DoomVertex dmvert[2*1024];
DoomLine dmline[2*1024];
DoomSec dmsec[2*1024];
DoomSsec dmssec[2*1024];
DoomSeg dmseg[2*1024];
DoomNode dmnode[2*1024];
DoomSide dmside[2*1024];
int dmlinecnt;
int dmnodecnt;

float myx=1087;
float myy=-3504;
float myz=-34;
float myangle=0;
float sc=(1.0/4.0);

#define KEY_UP 1
#define KEY_DOWN 2
#define KEY_LEFT 4
#define KEY_RIGHT 8
int keys=0;

float proj[4][4];
float view[4][4];

cairo_t *myc;
cairo_font_face_t *font;

typedef struct {
	DoomVertex p[2];
} Line2d;

#define LINEMAX 150
Line2d lines[LINEMAX];
int lineIdx;

void linesClear() {
	lineIdx=0;
}

void lineAdd(int xs, int ys, int xe, int ye) {
	int t;
	if (xs>xe) {
		t=xs; xs=xe; xe=t;
		t=ys; ys=ye; ye=t;
	}
	lines[lineIdx].p[0].x=xs;
	lines[lineIdx].p[0].y=ys;
	lines[lineIdx].p[1].x=xe;
	lines[lineIdx++].p[1].y=ye;
}

void linesDraw(cairo_t *c) {
	int i, j, k, d,dx, dy;
	int cd, ci, inv;
	int x=SIZEX/2,y=SIZEY/2;
	for (i=0; i<lineIdx; i++) {
		//Find closest line
		cd=9999; ci=0;
		for (j=0; j<lineIdx; j++) {
			if (lines[j].p[0].x!=-1) {
				for (k=0; k<2; k++) {
					dx=abs(lines[j].p[k].x-x);
					dy=abs(lines[j].p[k].y-y);
					if (dx>dy) d=dx; else d=dy;
					if (d<cd) {
						cd=d;
						ci=j;
						inv=k;
					}
				}
			}
		}
//		cairo_move_to(c, x1, y1);
		cairo_line_to(c, lines[ci].p[inv].x, lines[ci].p[inv].y);
		cairo_line_to(c, lines[ci].p[inv^1].x, lines[ci].p[inv^1].y);
		lines[ci].p[0].x=-1; //disable line
		x=lines[ci].p[inv^1].x;
		y=lines[ci].p[inv^1].y;
		if (mode&3) cairo_stroke(c);
	}
}

void m4m4Mult(float m1[4][4], float m2[4][4], float res[4][4]) {
	int x, y, p;
	float c;
	for (x=0; x<4; x++) {
		for (y=0; y<4; y++) {
			c=0;
			for (p=0; p<4; p++) {
				c+=m1[p][x]*m2[y][p];
			}
			res[y][x]=c;
		}
	}
}

void vm4Mult(float vc[4], float m[4][4], float res[4]) {
	int u, v;
	float c;
	for (u=0; u<4; u++) {
		c=0;
		for (v=0; v<4; v++) {
			c+=m[u][v]*vc[v];
		}
		res[u]=c;
	}
}

//Cross-product the first 3 dimensions of a and b. Res[4]=1.
void crossProd(float a[4], float b[4], float res[4]) {
	res[0]=a[1]*b[2]-a[2]*b[1];
	res[1]=a[2]*b[0]-a[0]*b[2];
	res[2]=a[0]*b[1]-a[1]*b[0];
	res[3]=1;
}

void genProjMat(float fov, float aspect, float near, float far) {
	//Generate the relevant bits of a projection matrix
	// http://stackoverflow.com/questions/18404890/how-to-build-perspective-projection-matrix-no-api
	// http://rodrigo-silveira.com/3d-programming-transformation-matrix-tutorial/
	int i;
	for (i=0; i<16; i++) proj[i/4][i%4]=0;
	float depth=far-near;
	float cotA2=1.0/tan(0.5f*fov);
	proj[0][0]=cotA2*aspect;
	proj[1][1]=cotA2*(1.0/aspect);
	proj[2][2]=(-near-far)/depth;
	proj[2][3]=2*far*near/depth;
	proj[3][2]=1;
}

void genViewMat() {
	float tmp[4][4];
	float rot[4][4]={
		{cos(myangle), 0, sin(myangle), 0},
		{0,1,0, 0},
		{-sin(myangle), 0, cos(myangle), 0},
		{0,0,0,1}
	};
	float scale[4][4]={
		{sc,0,0,0},
		{0,sc,0,0},
		{0,0,sc,0},
		{0,0,0,1}
	};
	float trans[4][4]={
		{1,0,0,-myx},
		{0,1,0,-myz},
		{0,0,1,-myy},
		{0,0,0,1}
	};
	m4m4Mult(trans, rot, tmp);
	m4m4Mult(tmp, scale, view);
}

void corrvformap(float in[4], float ret[4]) {
	ret[0]=in[0]/1000;
	ret[1]=-in[2]/1000;
	ret[2]=in[1]/1000;
	ret[3]=in[3];
}

void projectv(float x, float y, float z, float ret[4]) {
	float in[4]={x,y,z,1};
	vm4Mult(in, view, ret);
}

void projectvtop(float in[4], float ret[4]) {
	int j;
	vm4Mult(in, proj, ret);
	for (j=0; j<3; j++) {
		ret[j]/=ret[3];
	}
}


//Applies the view and projection matrix; does a divide-by-W.
void project(float x, float y, float z, float ret[4]) {
	float tmp[4];
	projectv(x,y,z,tmp);
	projectvtop(tmp,ret);
}


int in1(float r[4]) {
	int i;
	for (i=0; i<3; i++) {
		if (r[i]>1 || r[i]<-1) return 0;
	}
	if (r[3]<0) return 0;
	return 1;
}


void drawLine(cairo_t *c, int x1, int y1, int x2, int y2) {
	lineAdd(x1,y1,x2,y2);
	return;
	cairo_move_to(c, x1, y1);
//	cairo_line_to(c, x1, y1);
	cairo_line_to(c, x2, y2);
	if (mode&3) cairo_stroke(c);
}

unsigned int zbm[SIZEX*SIZEY/32];

void zbmClear() {
	int i;
	for (i=0; i<(SIZEX*SIZEY)/32; i++) zbm[i]=0;
}

void zbmVLine(int x, int y1, int y2) {
	int y;
	//Return if x is out of bounds
	if (x<0 || x>=SIZEX) return;
	//Re-order y1 and y2 if needed
	if (y1>y2) {
		y=y2; y2=y1; y1=y;
	}
	//Clip values
	if (y1<0) y1=0;
	if (y2<0) y2=0;
	if (y1>=SIZEY) y1=SIZEY-1;
	if (y2>=SIZEY) y2=SIZEY-1;
	//ToDo: Optimize for 32-bit moves if line len is
	//long enough.
	for (y=y1; y<=y2; y++) {
		zbm[(x*SIZEY+y)/32]|=(1<<(y&31));
	}
}

int zbmPixSet(int x, int y) {
	if (mode&1) return 0;
	if (x<0 || y<0 || x>=SIZEX || y>=SIZEY) return 1;
	return zbm[(x*SIZEY+y)/32] & (1<<(y&31));
}

void drawVertLine(cairo_t *c, int x, int yl, int yh) {
	int drawingLine=0;
	int ly, y;
	for (y=yl; y<yh; y++) {
		if (zbmPixSet(x, y)) {
			if (drawingLine) {
				drawingLine=0;
				drawLine(c, x, ly, x, y-1);
			}
		} else {
			if (!drawingLine) {
				drawingLine=1;
				ly=y;
			}
		}
	}
	if (drawingLine) {
		drawLine(c, x, ly, x, y-1);
	}
}

//Draw (mostly) horizontal line
void drawHorLine(cairo_t *c, int x1, int y1, int x2, int y2) {
	int x,y;
	int drawingLine=0;
	int lx, ly, ex, ey;
	int dx=x2-x1;
	int dy=y2-y1;
	if (dx==0) return;
	for (x=x1; x<=x2; x++) {
		y=y1+((dy*(x-x1))/dx);
		if (zbmPixSet(x, y)) {
			if (drawingLine) {
				drawingLine=0;
				drawLine(c, lx, ly, ex, ey);
			}
		} else {
			if (!drawingLine) {
				drawingLine=1;
				lx=x; ly=y;
			}
			ex=x; ey=y;
		}
	}
	if (drawingLine) {
		drawLine(c, lx, ly, ex, ey);
	}
}


void drawWall(cairo_t *c, int x1, int y1, int x2, int y2, int zs, int ze, int flats, int ends) {
	float vba[4], vbb[4], vta[4], vtb[4];
	float ba[4], bb[4], ta[4], tb[4];
	int rx1, rx2, ryh1, ryh2, ryl1, ryl2;
	int x, t;
	int dx, dyh, dyl;
	int cyh, cyl;
	if (zs==ze) return; //no need to draw no-height walls
	if (zs>ze) {
		t=zs; zs=ze; ze=t;
	}

	//Convert linesegs to view first
	projectv(x1, -zs, y1, vba);
	projectv(x2, -zs, y2, vbb);
	projectv(x1, -ze, y1, vta);
	projectv(x2, -ze, y2, vtb);

	//Cut off walls that go behind us.
#if 1
	if (vba[2]<NEARPT && vbb[2]<NEARPT) return; //can't see line anyway
	if (vba[2]<NEARPT || vbb[2]<NEARPT) {
		//Line crosses view point. Calculate where.
		int cutx=vba[0]+(vbb[0]-vba[0])*((vba[2]-NEARPT)/(vba[2]-vbb[2]));
		if (vba[2]<NEARPT) {
			vba[2]=NEARPT;
			vba[0]=cutx;
			vta[2]=NEARPT;
			vta[0]=cutx;
		} else {
			vbb[2]=NEARPT;
			vbb[0]=cutx;
			vtb[2]=NEARPT;
			vtb[0]=cutx;
		}
	}
#endif

	if (mode&4) {
		corrvformap(vba, ba);
		corrvformap(vbb, bb);
		corrvformap(vta, ta);
		corrvformap(vtb, tb);
	} else {
		projectvtop(vba, ba);
		projectvtop(vbb, bb);
		projectvtop(vta, ta);
		projectvtop(vtb, tb);
	}


	rx1=ba[0]*SIZEX+SIZEX/2;
	rx2=bb[0]*SIZEX+SIZEX/2;
	ryl1=ta[1]*SIZEY+SIZEY/2;
	ryl2=tb[1]*SIZEY+SIZEY/2;
	ryh1=ba[1]*SIZEY+SIZEY/2;
	ryh2=bb[1]*SIZEY+SIZEY/2;

	if (rx1==rx2) return; //disregard walls we're looking at on a 90 degrees angle

	cairo_set_source_rgb(c, 0,0,0);
	if (rx1<rx2) {
		if (ends&1) drawVertLine(c, rx1, ryl1, ryh1);
		if (ends&2) drawVertLine(c, rx2, ryl2, ryh2);
		if (mode&2) {
			drawHorLine(c, rx1, ryh1, rx2, ryl2);
			drawHorLine(c, rx1, ryl1, rx2, ryh2);
		}
		drawHorLine(c, rx1, ryh1, rx2, ryh2);
		drawHorLine(c, rx1, ryl1, rx2, ryl2);
	} else {
		//We're looking at the _back_ of a segment. Don't draw this as a wall.
		//Do draw the top or bottom lines if we need no fill there!

		//...but first, swap around coords because otherwise we can't draw
		//them at all :X
		t=rx2; rx2=rx1; rx1=t;
		t=ryh2; ryh2=ryh1; ryh1=t;
		t=ryl2; ryl2=ryl1; ryl1=t;

		if (flats==1) drawHorLine(c, rx1, ryh1, rx2, ryh2);
		if (flats==-1) drawHorLine(c, rx1, ryl1, rx2, ryl2);


		if (mode&2) {
			cairo_set_source_rgb(c, 0, 1, 0);
			drawVertLine(c, rx1, ryl1, ryh1);
			drawVertLine(c, rx2, ryl2, ryh2);
			drawHorLine(c, rx1, ryh1, rx2, ryl2);
			drawHorLine(c, rx1, ryl1, rx2, ryh2);
			drawHorLine(c, rx1, ryh1, rx2, ryh2);
			drawHorLine(c, rx1, ryl1, rx2, ryl2);
			drawHorLine(c, rx1, ryh1, rx2, ryh2);
			drawHorLine(c, rx1, ryl1, rx2, ryl2);
		}
	}

	dx=(rx2-rx1);
	//Flats is 0 if ceiling and floor should be covered, -1 if floor and 1 if ceiling must be covered.
	if (flats>=0) {
		ryl1=0;
		ryl2=0;
	}
	if (flats<=0) {
		ryh1=SIZEY;
		ryh2=SIZEY;
	}

	dyh=(ryh2-ryh1);
	dyl=(ryl2-ryl1);
	for (x=rx1; x<=rx2; x++) {
		cyh=ryh1+((dyh*(x-rx1))/dx);
		cyl=ryl1+((dyl*(x-rx1))/dx);
		zbmVLine(x, cyl, cyh);
//		if ((x&3)==0)drawLine(c, x, cyl, x, cyh);
	}
}


void drawSsec(cairo_t *c, int ssec) {
	int i;
	DoomVertex *v1, *v2;
	DoomLine *l;
	int secR, secL, t, endl=0, endr=0;
	for (i=dmssec[ssec].sStart; i<dmssec[ssec].sStart+dmssec[ssec].sCount; i++) {
		v1=&dmvert[dmseg[i].vStart];
		v2=&dmvert[dmseg[i].vEnd];
		l=&dmline[dmseg[i].linedef];
		
		if (v1->x==dmvert[l->vFrom].x && v1->y==dmvert[l->vFrom].y) endr=1;
		if (v2->x==dmvert[l->vTo].x && v2->y==dmvert[l->vTo].y) endl=1;
		
//		printf("Drawing ssec %d seg %d: linedef %d (l %d r %d)\n", ssec, i, dmseg[i].linedef,  l->sdRight, l->sdLeft);
		secR=dmside[l->sdRight].sector;
		if (l->sdLeft==-1) {
			drawWall(c, v1->x, v1->y, v2->x, v2->y, dmsec[secR].zFloor, dmsec[secR].zCeil, 0, (endl<<1)+endr);
		} else {
			secL=dmside[l->sdLeft].sector;
			if (dmseg[i].dir) {
				//Seg runs opposite to line. Switch secL and secR so we're in the direction
				//assumed by the seg.
				t=secL; secL=secR; secR=t;
			}
			if (dmsec[secR].zFloor>dmsec[secL].zFloor) {
				drawWall(c, v2->x, v2->y, v1->x, v1->y, dmsec[secL].zFloor, dmsec[secR].zFloor, -1, (endr<<1)+endl);
			} else {
				drawWall(c, v1->x, v1->y, v2->x, v2->y, dmsec[secR].zFloor, dmsec[secL].zFloor, -1, (endl<<1)+endr);
			}
			if (dmsec[secR].zCeil<dmsec[secL].zCeil) {
				drawWall(c, v2->x, v2->y, v1->x, v1->y, dmsec[secL].zCeil, dmsec[secR].zCeil, 1, (endr<<1)+endl);
			} else {
				drawWall(c, v1->x, v1->y, v2->x, v2->y, dmsec[secR].zCeil, dmsec[secL].zCeil, 1, (endl<<1)+endr);
			}
		}
	}
	
}


void numAt(int x, int y, int z, int n) {
	char buff[10];
	float p[4], pn[4];
	sprintf(buff, "%d", n);
	if (mode&4) {
		projectv(x,-z,y,pn);
		corrvformap(pn, p);
	} else {
		project(x,-z,y,p);
	}
	cairo_select_font_face(myc, "sans-serif", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
	cairo_set_font_size(myc, 10);
	cairo_move_to(myc, p[0]*SIZEX+SIZEX/2, p[1]*SIZEY+SIZEY/2);
	cairo_show_text(myc, buff);
}

//Fuck it. Ripped from http://stackoverflow.com/questions/563198/how-do-you-detect-where-two-line-segments-intersect
int get_line_intersection(float p0_x, float p0_y, float p1_x, float p1_y, 
    float p2_x, float p2_y, float p3_x, float p3_y, float *i_x, float *i_y)
{
    float s02_x, s02_y, s10_x, s10_y, s32_x, s32_y, s_numer, t_numer, denom, t;
    s10_x = p1_x - p0_x;
    s10_y = p1_y - p0_y;
    s32_x = p3_x - p2_x;
    s32_y = p3_y - p2_y;

    denom = s10_x * s32_y - s32_x * s10_y;
    if (denom == 0)
        return 0; // Collinear
    int denomPositive = denom > 0;

    s02_x = p0_x - p2_x;
    s02_y = p0_y - p2_y;
    s_numer = s10_x * s02_y - s10_y * s02_x;
    if ((s_numer < 0) == denomPositive)
        return 0; // No collision

    t_numer = s32_x * s02_y - s32_y * s02_x;
    if ((t_numer < 0) == denomPositive)
        return 0; // No collision

    if (((s_numer > denom) == denomPositive) || ((t_numer > denom) == denomPositive))
        return 0; // No collision
    // Collision detected
    t = t_numer / denom;
    if (i_x != NULL)
        *i_x = p0_x + (t * s10_x);
    if (i_y != NULL)
        *i_y = p0_y + (t * s10_y);

    return 1;
}

int currNode;

//Check if contents within a bounding box should be visible.
int bbVisible(DoomBbox *bb) {
	float vp[4][4];
	int j;
	int flag=0;
	//Every time we look one unit further, the fov will deviate dfov units
	//more from the center of view.
	static float dfov=-1;
	if (dfov==-1) dfov=sin(FOV/2.0)/cos(FOV/2.0);

	projectv(bb->xlo, 0, bb->ylo, vp[0]);
	projectv(bb->xhi, 0, bb->ylo, vp[1]);
	projectv(bb->xhi, 0, bb->yhi, vp[2]);
	projectv(bb->xlo, 0, bb->yhi, vp[3]);

	//Discard if entirely behind player
	flag=0;
	for (j=0; j<4; j++) {
		if (vp[j][2]>=NEARPT) flag=1;
	}
	if (!flag) return 0;
	//Discard if entirely too far away
	flag=0;
	for (j=0; j<4; j++) {
		if (vp[j][2]<FARPT) flag=1;
	}
	if (!flag) return 0;

	//Any bboxes left are in the 'butter zone': not too far away, not
	//too near. We still have a huge horizontal range to deal with. We can strip away
	//all bboxes that fall outside the fov too.
	flag=0;
	for (j=0; j<4; j++) {
		if ((vp[j][2]*dfov)>vp[j][0]) flag=1;
	}
	if (!flag) return 0;

	flag=0;
	for (j=0; j<4; j++) {
		if ((vp[j][2]*-dfov)<vp[j][0]) flag=1;
	}
	if (!flag) return 0;

	return 1;
}

void renderBspNode(cairo_t *c, int node, int *col) {
	if (node&0x8000) {
		//Node is a SSECTOR. Draw it.
		node&=0x7fff;
		(*col)++;
		cairo_set_source_rgb(c, 1-*col/128.0, 0, *col/128.0);
		drawSsec(c, node);
	} else {
		float cp[4], toMe[4], ld[4];
		//Calculate cross product of node line and vector to player pos to
		//figure out if player is to the left or to the right of node line
		toMe[0]=dmnode[node].lineStart.x-myx;
		toMe[1]=dmnode[node].lineStart.y-myy;
		toMe[2]=0;
		ld[0]=dmnode[node].lineD.x;
		ld[1]=dmnode[node].lineD.y;
		ld[2]=0;
		crossProd(toMe, ld, cp);
		//First draw front, then back node.
		currNode=node;
		if (cp[2]>0) {
			currNode=node;
			if (bbVisible(&dmnode[node].bbLeft)) renderBspNode(c, dmnode[node].lChild, col);
			currNode=node;
			if (bbVisible(&dmnode[node].bbRight)) renderBspNode(c, dmnode[node].rChild, col);
		} else {
			currNode=node;
			if (bbVisible(&dmnode[node].bbRight)) renderBspNode(c, dmnode[node].rChild, col);
			currNode=node;
			if (bbVisible(&dmnode[node].bbLeft)) renderBspNode(c, dmnode[node].lChild, col);
		}
	}
}



void redraw(cairo_t *c) {
	int col=0;
	cairo_new_path(c);
	cairo_rectangle(c, 0, 0, SIZEX, SIZEY);
	cairo_set_source_rgb(c, 1.0, 1.0, 1.0);
	cairo_fill(c);

	//Draw crosshair
	cairo_set_source_rgb(c, 0, 0, 0);
	cairo_set_line_width(c, 1);
	cairo_move_to(c, -5+SIZEX/2, SIZEY/2);
	cairo_line_to(c, 5+SIZEX/2, SIZEY/2);
	cairo_stroke(c);
	cairo_move_to(c, SIZEX/2, -5+SIZEY/2);
	cairo_line_to(c, SIZEX/2, 5+SIZEY/2);
	cairo_stroke(c);

	genViewMat();
	zbmClear();
	linesClear();
	renderBspNode(c, dmnodecnt-1, &col);
	linesDraw(c);
	cairo_stroke(c);
	cairo_show_page(c);
	printf("--------------\n");
}


void loadStuff() {
	FILE *f;
	int n;

	f=fopen("e1l1/VERTEXES", "r");
	n=fread(dmvert, sizeof(DoomVertex), 2*1024, f);
	printf("Read %d vertices\n", n);
	fclose(f);

	f=fopen("e1l1/LINEDEFS", "r");
	n=fread(dmline, sizeof(DoomLine), 2*1024, f);
	printf("Read %d linedefs\n", n);
	dmlinecnt=n;
	fclose(f);

	f=fopen("e1l1/SECTORS", "r");
	n=fread(dmsec, sizeof(DoomSec), 2*1024, f);
	printf("Read %d sectors\n", n);
	fclose(f);

	f=fopen("e1l1/SSECTORS", "r");
	n=fread(dmssec, sizeof(DoomSsec), 2*1024, f);
	printf("Read %d ssectors\n", n);
	fclose(f);

	f=fopen("e1l1/SEGS", "r");
	n=fread(dmseg, sizeof(DoomSeg), 2*1024, f);
	printf("Read %d segs\n", n);
	fclose(f);

	f=fopen("e1l1/SIDEDEFS", "r");
	n=fread(dmside, sizeof(DoomSide), 2*1024, f);
	printf("Read %d sides\n", n);
	fclose(f);

	f=fopen("e1l1/NODES", "r");
	n=fread(dmnode, sizeof(DoomNode), 2*1024, f);
	printf("Read %d nodes\n", n);
	fclose(f);
	dmnodecnt=n;
}


int main(int argc, char **argv) {
	Display *dpy;
	Window rootwin, win;
	XEvent e, ne;
	int scr;
	cairo_surface_t *cs;
	cairo_t *c;

	fontInit();
	font=fontLoad("/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans.ttf", 0);
	genProjMat(FOV, (float)SIZEY/(float)SIZEX, 15.0, 250.0);
	loadStuff();
	if (!(dpy=XOpenDisplay(NULL))) {
		printf("Can't open X display.\n");
		exit(1);
	}
	scr=DefaultScreen(dpy);
	rootwin=RootWindow(dpy, scr);
	win=XCreateSimpleWindow(dpy, rootwin, 1, 1, SIZEX, SIZEY, 0, BlackPixel(dpy, scr), BlackPixel(dpy, scr));
	XStoreName(dpy, win, "3d test");
	XSelectInput(dpy, win, ExposureMask|KeyPressMask|KeyReleaseMask);
	XMapWindow(dpy, win);

	cs=cairo_xlib_surface_create(dpy, win, DefaultVisual(dpy, 0), SIZEX, SIZEY);
	c=cairo_create(cs);
	myc=c;
	while(1) {
		float dist;
		while (XPending(dpy)) {
			XNextEvent(dpy, &e);
			if (e.type==Expose && e.xexpose.count<1) {
				redraw(c);
			}
			if (e.type==KeyPress || e.type==KeyRelease) {
				//Ignore autorepeat releases
				if (XEventsQueued(dpy, QueuedAfterReading)) {
					XPeekEvent(dpy, &ne);
					if (e.type==KeyRelease && ne.type == KeyPress && ne.xkey.time == e.xkey.time && ne.xkey.keycode == e.xkey.keycode) continue;
				}
				XKeyEvent k=e.xkey;
				if (k.keycode==9 && e.type==KeyPress) exit(0); //esc pressed
				if (k.keycode==111) { if (e.type==KeyPress) keys|=KEY_UP; else keys&=~KEY_UP; }
				if (k.keycode==116) { if (e.type==KeyPress) keys|=KEY_DOWN; else keys&=~KEY_DOWN; }
				if (k.keycode==113) { if (e.type==KeyPress) keys|=KEY_LEFT; else keys&=~KEY_LEFT; }
				if (k.keycode==114) { if (e.type==KeyPress) keys|=KEY_RIGHT; else keys&=~KEY_RIGHT; }
				if (k.keycode==38 && e.type==KeyPress) myz+=3;
				if (k.keycode==52 && e.type==KeyPress) myz-=3;
				if (k.keycode==65 && e.type==KeyPress) mode++;
//				printf("%d\n", k.keycode);
			}
		}
		//Movement
		dist=0;
		if (keys&KEY_UP) dist=20;
		if (keys&KEY_DOWN) dist=-20;
		if (keys&KEY_LEFT) myangle+=0.06;
		if (keys&KEY_RIGHT) myangle-=0.06;
		myx+=sin(-myangle)*dist;
		myy+=cos(-myangle)*dist;
//		printf("%f %f %f\n", myx, myy, myz);
		redraw(c);
		usleep(40000);
	}
}
