; Copyright (C) 2015 Jeroen Domburg <jeroen at spritesmods.com>
;
; This library is free software: you can redistribute it and/or modify
; it under the terms of the GNU Lesser General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This library is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU Lesser General Public License for more details.
;
; You should have received a copy of the GNU Lesser General Public License
; along with this library.  If not, see <http://www.gnu.org/licenses/>.

	include "vectrex.i"

vecpos	EQU	$ca00
type	EQU $ca02
tmp		EQU	$ca04
rpcfn	EQU $cb00

;Cartridge header
	ORG 0
	fcb "g GCE 2015", $80		;'g' is copyright sign
	fdb music1					;music from the rom
	fcb $F8, $50, $20, -$56		; height, width, rel y, rel x (from 0,0)
	fcb "BAD APPLE",$80; some game information ending with $80
	fcb 0		;				 end of game header

;The Sound Issue...
;If we want to send out sound at 11KHz and the frame rate is 60Hz, we have to send
;183 samples per frame. (Twice that per frame of video.)
;It also means we have 1363 CPU cycles per sample... Hmm. With a line taking as little as
;(max) 255 samples, it should mean we can interpolate using the frame rate counter, sending 
;a new sample every time weve drawn a line, and continuously in the vblank interval.

;The frame rate counter counts down from $61A8 if the animation runs at 30FPS.

;So what do we do? (61a8<<2)>>8 gives 390, which gives a nice sample rate of
;11700Hz. 

main
;copy rpc fn into place
	ldx #rpcfndat
	ldy #rpcfn
rpccopyloop
	lda ,x+
	sta ,y+
	cmpx #rpcfndatend
	bne rpccopyloop

	;set dp to $d0
	lda #$d0
	tfr a,dp
	setdp #$d0

	;Medium intensity
	JSR Intensity_3F

restart
	jmp loadandgo

loop
	jsr doaudio
	;Medium intensity
	JSR Intensity_3F
	jsr doaudio

;	jmp drawdone		;Enable this for audio-only
	LDA #$18			;scale
	STA VIA_t1_cnt_lo
	LDX vecpos
drawmore

	;We're at the start of a new vector.
	;Format: flags count y1 x1 [y2 x2 [...]]
	lda ,x+ ;Grab flags
	sta type ;Store for later use
	sta 0
	cmpa #$20 ;No need to draw anything?
	beq drawdone
	anda #$f ;Get initial move iterations.
	cmpa #$0 ;No move needed?
	beq nomove

moveloop
	;We need to move the cursor a times to the thing indicated by x
 IF 0
	pshs a
	lda 1,x ;Grab 1st Y-coord
	ldb 2,x ;Grab 1st X-coord
	jsr Moveto_d ;Do move
	puls a
 ELSE
	;Load and set Y coord
	ldb 1,x ;xcoord
	stb VIA_port_a ;aka DAC
	clr VIA_port_b ;aka CNTRL; enable s/h strobe
	ldb #$ce 		;PUNZRO
	stb VIA_cntl ;aka PCNTRL
	ldb #$00
	stb VIA_shift_reg
	inc VIA_port_b ;disable s/h
	ldb 2,x ;y coord
	stb VIA_port_a
	;Go draw; wait till done.
	clr VIA_t1_cnt_hi
	ldb #$40
movewait
	bitb VIA_int_flags
	beq movewait
 ENDIF
	jsr doaudio

	deca
	bne moveloop ;Loop more?


	lda 0,x ;Grab coord count
	leax 3,x ;Skip 1st coord and count
	bra dodraw ;Go draw next coords
nomove
	lda ,x+ ;grab draw count
	inca ;count 0 means 1
dodraw
 IF 1
	;Load and set Y coord
	ldb ,x+ ;x coord
	stb VIA_port_a
	clr VIA_port_b ;enable s/h strobe
	ldb ,x+ ;y coord
	nop
	inc VIA_port_b ;disable s/h
	stb VIA_port_a
	ldb #$FF
	stb VIA_shift_reg
	;Go draw; wait till done.
	clr VIA_t1_cnt_hi
	ldb #$40
drawwait
	bitb VIA_int_flags
	beq drawwait
	;We're done.
	clr VIA_shift_reg

	;Check if we need to handle the audio.
	jsr doaudio
	;See if we need to draw more vectors
	deca
	bne dodraw
 ELSE
	deca
	jsr Draw_VL_a
 ENDIF
	
	lda type
	anda #$f0
	cmpa #$f0 ;restart token?
	bne norestart
	jmp restart
norestart

	cmpa #$10	;Need to draw more shit?
	bne drawdone
	jmp drawmore
drawdone
	;We're done with this frame.

;Wait for recal
 IF 0
	jmp Wait_Recal
 ENDIF
	ldb 0
wrecloop
	incb
	bitb #3
	bne noaudiorecal
	jsr doaudio
noaudiorecal
	;Check if vblank has ended by now
	;Can't use the via int flag because reading t2_lo resets that.
	lda VIA_t2_hi
	anda #$e0
	cmpa #$e0
	bne wrecloop

	ldd #$50c3 ;30fps
	std VIA_t2_lo
	jsr Recalibrate

loadandgo
	;Load next frame over rpc
	lda #2
	jsr rpcfn
	;Next frame starts at 0x4000
	ldx #$4000
	stx vecpos
	ldy #$4400 ;audio ptr
	jsr doaudio
	jmp loop

;Handle audio
doaudio
	pshs d
	;Get correct sample location
	ldd VIA_t2_lo ;/load refresh register. WARNING: hi and lo is switched around (a lo, b hi)
	cmpb #$c4
	bhi noaudio
	std 1
	
	;Output sound
	lda ,y+
	sta VIA_port_a
	lda #$6		;00000110
	sta VIA_port_b
	nop
	nop
	nop
	nop
	nop
	nop
	lda #$1		;00000001
	sta VIA_port_b
noaudio
	puls d
	rts

;Rpc function. Because this makes the cart unavailable, this
;will copied to SRAM. Call as rpcfn.
rpcfndat
	sta $7fff
rpcwaitloop
	lda $0
	cmpa #'g'
	bne rpcwaitloop
	lda $1
	cmpa #' '
	bne rpcwaitloop
	rts
rpcfndatend



	;zero integrators. WARNING: Moves shit back to (0,0)
;	ldb #$cc
;	stb VIA_cntl
;	clr VIA_shift_reg
