#!/usr/bin/php
<?php
//Script to combine GeoJSON images into multiple Vectrex vector lists
/*
 *  Copyright (C) 2015 Jeroen Domburg <jeroen at spritesmods.com>
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

//Amount of frames in the movie
$framect=6570;



//Takes vectors and input filename and generates an image with the two superposed, so
//you can check the image to see if the vectors are traced/simplified in an acceptable
//fashion.
function drawTestImage($pix, $out, $scale) {
	$mysc=0.4;
	$ofn=sprintf("tst/%06d.png", $out);
	$ifn=sprintf("png/%08d.png", $out);
	$im=imagecreatetruecolor(256*$scale*$mysc,256*$scale*$mysc);
	$b=imagecolorallocate($im, 0,0,0);
//	$c[1]=imagecolorallocate($im, 255,0,0);
//	$c[0]=imagecolorallocate($im, 0,255,0);
	imagealphablending($im, true);
	$c[1]=imagecolorallocatealpha($im, 0,128,255,100);
	$c[0]=imagecolorallocatealpha($im, 0,128,255,100);
	$bim=imagecreatefrompng($ifn);
	if ($bim!==FALSE) {
		imagecopyresized($im, $bim, 0, 0, 0, 0, 256*$scale*$mysc,256*$scale*$mysc,imagesx($bim), imagesy($bim));
		imagedestroy($bim);
	}
	$px=128*$scale;
	$py=128*$scale;
	$n=0;
	foreach($pix as $l) {
		$first=true;
		foreach($l["c"] as $p) {
			$x=intval($p[0]);
			$y=intval($p[1]);
			if ($first && $l["t"]!=0) {
				$x*=$l["t"]; $y*=$l["t"];
				$x+=$px; $y+=$py;
			} else {
				$x+=$px; $y+=$py;
				//Draw thicker line...
				for ($nx=-1; $nx!=2; $nx++) {
				for ($ny=-1; $ny!=2; $ny++) {
				imageline($im,$px*$mysc+$nx,((255*$scale)-$py)*$mysc+$ny,$x*$mysc+$nx,((255*$scale)-$y)*$mysc+$ny,$c[$n&1]);
				}
				}
			}
			$first=false;
			$px=$x; $py=$y;
		}
		$n++;
	}
	imagepng($im, $ofn);
	imagedestroy($im);
}



$bytePos=0;
//Output a comment to an include file.
function outComment($c) {
	global $bytepos;
	return;
	echo "\n\t\t\t;$c";
	$bytepos=($bytepos+15)&(~15);
}

//Output a byte to an include file.
function outByte($b) {
	global $bytepos;
	if ($bytepos==0) {
		echo "\nveclists\tfcb ";
	} else {
		if (($bytepos&15)==0) {
			echo "\n\t\t\tfcb ";
		} else {
			echo ", ";
		}
	}
	printf("$%02x", $b&0xff);
	$bytepos++;
	if ($b<-128 || $b>255) {
		outComment("Prev byte overflowed! Real value ".$b);
	}
}


//Converts GeoJSON to internal data
//In: JSON data grabbed from the GeoJSON file
//Out: array of polys with abs coordinates
function grabPolys($data, $isx, $isy) {
	//Grab poly info and convert to (255,255)-sized grid
	if (count($data->features)==0) return array(); 
	$poly=$data->features[0]->geometry->coordinates[0];
	$outp=array();
	$x=0; $y=0;
	foreach($data->features as $poly) {
		foreach($poly->geometry->coordinates as $coords) {
			$out=array();
			foreach($coords as $p) {
				$rx=(($p[0]*255)/$isx);
				$ry=(($p[1]*255)/$isy);
				$out[]=array($rx, $ry);
			}
			$outp[]=$out;
		}
	}
	return $outp;
}

//Gets rid of lines along the borders of the image
//In/Out: array of polys with abs coordinates
function cullBorders($in) {
	$out=array();
	foreach($in as $poly) {
		$outp=array();
		foreach($poly as $coord) {
			list($x, $y)=$coord;
			if ($x<=1 || $x>=254 || $y<=1 || $y>=254) {
				$out[]=$outp;
				$outp=array();
			}
			$outp[]=array($x, $y);
		}
		$out[]=$outp;
	}
	return $out;
}

//Calculate the angle between two lines, in rads
function calcAngle($x1, $y1, $x2, $y2, $x3, $y3) {
	$dv1x=$x2-$x1; $dv1y=$y2-$y1;
	$dv2x=$x2-$x3; $dv2y=$y2-$y3;
	$n1=sqrt(($dv1x*$dv1x)+($dv1y*$dv1y));
	$n2=sqrt(($dv2x*$dv2x)+($dv2y*$dv2y));
	$np=$n1*$n2;
	$dp=($dv1x*$dv2x)+($dv1y*$dv2y);
	if (abs($np)<0.0001) return M_PI/2;
//	var_dump(array($x1, $y1, $x2, $y2, $x3, $y3, $n1, $n2, $dp, $dp/$np));
	return acos($dp/$np);
}

//Optimize the polys to get rid of non-important lines
//In/Out: array of polys with abs coordinates
function optimizeMesh($in, $treshPix, $treshAngle) {
	$out=array();
	foreach($in as $n=>$poly) {
		$outp=array();
		$px=-9999;
		$py=-9999;
		foreach($poly as $n=>$coord) {
			list($x, $y)=$coord;
			$dx=$x-$px; $dy=$y-$py;
			$skip=true;
			//See if this line is really small.
			if (($dx*$dx)+($dy*$dy)>($treshPix*$treshPix)) $skip=false;
			//Calculate angle between this and next line
			$a=M_PI/2;
			if (($n+1)<count($poly)) {
				$a=calcAngle($px,$py, $x, $y, $poly[$n+1][0], $poly[$n+1][1]);
			}
//			echo "\nAngle=".($a*180/M_PI)."\n";
			if ($a<0) $a=-$a;
			if ($a>M_PI) $a=(2*M_PI)-$a;
			//If angle is small, we can merge this and the next line.
			if ($a<$treshAngle) $skip=false;
			if (!$skip) {
				//Add point to output array
				$outp[]=array($x, $y);
				$px=$x;
				$py=$y;
			}
		}
		$out[]=$outp;
	}
	return $out;
}


//Get rid of empty polys
//In/Out: array of polys with abs coordinates
function cullEmpty($in) {
	$out=array();
	foreach($in as $poly) {
		if (count($poly)>1) $out[]=$poly;
	}
	return $out;
}


//Compare array length; used for the sort in the limitBytes function
function cmpalen($a, $b) {
	if (count($a)==count($b)) return 0;
	return (count($a) > count($b)) ? -1 : 1;
}

//Limit the poly byte count by killing off smaller polys if needed
function limitBytes($outp, $max) {
	$res=array();
	$nb=0;
	usort($outp, "cmpalen");
	foreach($outp as $out) {
		$nb+=2+count($out)*2;
		if ($nb<$max) {
			$res[]=$out;
		} else {
			break;
		}
	}
	return $res;
}


//Convert to rel coords
//In: array of polys with abs coordinates
//Out: array of polys with rel coordinates
function scaleRel($in, $sc) {
	$out=array();
	$px=128*$sc;
	$py=128*$sc;
	foreach($in as $poly) {
		$outp=array();
		$first=true;
		foreach($poly as $coord) {
			list($x, $y)=$coord;
			$x*=$sc; $y*=$sc;	//Scale coords
			$dx=$x-$px; $dy=$y-$py; //Calculate distance
			//Calculate amount of moves we need to do
			$divx=intval(abs($dx/127))+1;
			$divy=intval(abs($dy/127))+1;
			$div=($divx>$divy)?$divx:$divy;
			//$div now contains the amount of steps to take.
			if ($first) {
				if ($dx==0 && $dy==0) {
					//Continuation of line. Leave out 1st coord
					$outp["t"]=0;
				} else {
					//Move of line
					$outp["t"]=$div;
					$outp["c"][]=array(intval($dx/$div), intval($dy/$div));
					$px+=intval($dx/$div)*$div;
					$py+=intval($dy/$div)*$div;
				}
			} else {
				//Just put multiple jumps in.
				for ($i=0; $i<$div; $i++) {
					$outp["c"][]=array(intval($dx/$div), intval($dy/$div));
					$px+=intval($dx/$div);
					$py+=intval($dy/$div);
				}
			}
			$first=false;
		}
		$out[]=$outp;
	}
	return $out;
}

//Divide polys that are >255 coords into multiple polys
function limitCntRel($in) {
	$out=array();
	foreach($in as $poly) {
		$outp=array();
		foreach($poly as $coord) {
			$outp[]=$coord;
			if (count($outp)==254) {
				$out[]=$outp;
				$outp=array($coord);
			}
		}
		if (count($outp)>1) $out[]=$outp;
	}
	return $out;
}


//Output vector list as inc file
//Output flag byte, poly len, poly coords.
//Flag byte:
//High nibble: flag
// - 0 - Frame end
// - 1 - Poly end but there are more polys in frame
// - 2 - Frame is empty
// - 0xf - Animation end
//Low byte: Initial move multiplicity
// - 0 - No move required. Draw from where you are to 1st coord in list
// - n - Do a relative move using 1st coords n times
function outputVecInc($outp, $last) {
	foreach($outp as $n=>$out) {
		outComment("poly $n, ".count($out["c"]). " coord pairs.");
		if ($n==(count($outp)-1)) {
			$f=($last?0xf:0);
		} else {
			$f=1;
		}
		outByte(($f<<4)+$out["t"]);
		outByte(count($out["c"])-1);
		foreach($out["c"] as $c) {
			outByte($c[1]/2);
			outByte($c[0]/2);
		}
	}
}


function countActions($in) {
	$n=0;
//	var_dump($in);
	foreach($in as $poly) {
		if ($poly["t"]!=0) $n+=$poly["t"]-1;
		$n+=count($poly["c"]);
	}
//	fprintf(STDERR, "Actions: %d\n", $n);
	return $n;
}

//Same as outputVecInc but in binary
function outputVecBin($outp, $last, $pad, $audio, $achunk, $apad, $atiming) {
	$no=0;
	if (count($outp)==0) {
		printf("%c", 0x20);
		$no=1;
	}
	foreach($outp as $n=>$out) {
		if ($n==(count($outp)-1)) {
			$f=($last?0xf:0); //last poly in frame or maybe anim
		} else {
			$f=1; //continued
		}
		printf("%c", (($f<<4)+$out["t"])); //output flag and move count
		printf("%c", count($out["c"])-1);
		$no+=2;
		foreach($out["c"] as $c) {
			printf("%c", $c[1]);
			printf("%c", $c[0]);
			$no+=2;
		}
	}
//	fprintf(STDERR,"%d\n", $no);
	while($no<$pad) {
		printf("%c", 0x5a);
		$no++;
	}
	$abin=fread($audio, $achunk);
	for ($x=0; $x<count($atiming); $x++) {
		$nat=(($x+1)==count($atiming))?$achunk+1:$achunk*$atiming[$x+1];
		$ss=$atiming[$x]*$achunk;
		$d=$nat-$ss;
//		fprintf(STDERR,"  d %d\n", $ss);
		$a=ord(substr($abin, $ss, 1));
		printf("%c", $a);
		$no++;
		if ($no>($pad+$apad)) {
			fprintf(STDERR, "SAMPLE OVERFLOW\n");
			exit(1);
			break;
		}
	}
	while($no<($pad+$apad)) {
		printf("%c", 0xa5);
		$no++;
	}
}

function getAudioTiming($f) {
	$tim=array();
	$oc=0;
	if ($f==NULL) return array();
	while (($l=fgets($f)) && !preg_match("/Read [0-9]+ bytes/", $l)) {
		if (preg_match("/DBG: 1 ([0-9a-fA-F]{2})([0-9a-fA-F]{2})/", $l, $m)) {
			$v=intval($m[2].$m[1], 16);
			if ($v>0xc350) {
				$oc++;
				$no=65535-$v;
				if ($oc>1) {
					fprintf(STDERR, "OVERFLOW DETECTED: %d\n", ($no));
					exit(1);
				}
			}
			$v=0xc350-$v;
			$tim[]=(floatval($v)/floatval(0xc350));
//			fprintf(STDERR, "%d\n", ($v*100)/0xc350);
		}
	}
//	fprintf(STDERR, "%d\n", count($tim));
	return $tim;
}


$framestart=1;
//$framestart=90;
//$frameend=1000;
$frameend=6570;

//$framestart=1886;
//$frameend=1888;

$audio=fopen("audiodump.raw", "rb");
$log=fopen("timing.log", "r");
getAudioTiming($log);

$scale=4;
for ($fn=$framestart; $fn<=$frameend; $fn++) {
	$file=sprintf("vec/%08d.vec", $fn);
	$data=json_decode(file_get_contents($file));
	$combinePixelLen=2;
	$combineAngle=170*(M_PI/180.0);
	do {
		$c=grabPolys($data, 480, 360);
		$c=cullBorders($c);
		$c=optimizeMesh($c, $combinePixelLen, $combineAngle);
		$c=cullEmpty($c);
		$c=scaleRel($c, $scale);
		$combinePixelLen+=2;
		$combineAngle-=5*(M_PI/180.0);
	} while (countActions($c)>280);

	outComment("Frame $fn");
//	outputVecInc($c, ($fn==$frameend));
	$audioTimings=getAudioTiming($log);
	outputVecBin($c, ($fn==$frameend), 1024, $audio, 390,  512, $audioTimings);
	if (!file_exists("tst")) mkdir("tst");
	drawTestImage($c, $fn, $scale);

}
echo "\n";

?>