/*
 *  Copyright (C) 2015 Jeroen Domburg <jeroen at spritesmods.com>
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <cairo/cairo.h>
#include <cairo/cairo-xlib.h>
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/select.h>
#include <time.h>
#include <sys/time.h>
#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdint.h>
#include "voom.h"

char lvlData[48*1024];

//Array to keep lines in
typedef struct {
	int x;
	int y;
} Vertex;

typedef struct {
	Vertex p[2];
} Line2d;

#define LINEMAX 150
Line2d lines[LINEMAX];
int lineIdx;


cairo_t *c;

void linesClear() {
	lineIdx=0;
}

void veLineAdd(int xs, int ys, int xe, int ye) {
	int t;
	if (lineIdx==LINEMAX) return;
	if (xs>xe) {
		t=xs; xs=xe; xe=t;
		t=ys; ys=ye; ye=t;
	}
	lines[lineIdx].p[0].x=xs;
	lines[lineIdx].p[0].y=ys;
	lines[lineIdx].p[1].x=xe;
	lines[lineIdx++].p[1].y=ye;
}

void linesDraw() {
	int i, j, k, d,dx, dy;
	int cd, ci, inv;
	int x=SIZEX/2,y=SIZEY/2;

	cairo_new_path(c);
	cairo_rectangle(c, 0, 0, SIZEX, SIZEY);
	cairo_set_source_rgb(c, 1.0, 1.0, 1.0);
	cairo_fill(c);
	cairo_set_source_rgb(c, 0, 0, 0);

//	printf("Drawing %i lines\n", lineIdx);
	for (i=0; i<lineIdx; i++) {
		//Find closest line
		cd=9999; ci=0;
		for (j=0; j<lineIdx; j++) {
			if (lines[j].p[0].x!=-1) {
				for (k=0; k<2; k++) {
					dx=abs(lines[j].p[k].x-x);
					dy=abs(lines[j].p[k].y-y);
					if (dx>dy) d=dx; else d=dy;
					if (d<cd) {
						cd=d;
						ci=j;
						inv=k;
					}
				}
			}
		}
		cairo_move_to(c, lines[ci].p[inv].x, lines[ci].p[inv].y);
		cairo_line_to(c, lines[ci].p[inv^1].x, lines[ci].p[inv^1].y);
		lines[ci].p[0].x=-1; //disable line
		x=lines[ci].p[inv^1].x;
		y=lines[ci].p[inv^1].y;
	}
	cairo_stroke(c);
}

void initEngine() {
	FILE *f;
	char *p=lvlData;
	const char *names[]={
		"e1l1/VERTEXES",
		"e1l1/LINEDEFS",
		"e1l1/SECTORS",
		"e1l1/SSECTORS",
		"e1l1/SEGS",
		"e1l1/SIDEDEFS",
		"e1l1/NODES"
	};
	char *dptr[7];
	int dsize[7];
	int n, i, l;
	l=sizeof(lvlData);
	for (i=0; i<7; i++) {
		f=fopen(names[i], "r");
		n=fread(p, 1, l, f);
		fclose(f);
		dptr[i]=p;
		dsize[i]=n;
		p+=n;
		l=l-n;
		printf("%s: %d bytes\n", names[i], n);
	}
	printf("Total level data size: %d bytes. %d bytes left in level buffer.\n", n, l);
	voomInit(dptr, dsize);
}


int main(int argc, char **argv) {
	Display *dpy=NULL;
	Window rootwin, win;
	XEvent e, ne;
	int scr;
	cairo_surface_t *cs;
	int keys=0;

	if (!(dpy=XOpenDisplay(NULL))) {
		printf("Can't open X display.\n");
		exit(1);
	}
	scr=DefaultScreen(dpy);
	rootwin=RootWindow(dpy, scr);
	win=XCreateSimpleWindow(dpy, rootwin, 1, 1, SIZEX, SIZEY, 0, BlackPixel(dpy, scr), BlackPixel(dpy, scr));
	XStoreName(dpy, win, "3d test");
	XSelectInput(dpy, win, ExposureMask|KeyPressMask|KeyReleaseMask);
	XMapWindow(dpy, win);

	cs=cairo_xlib_surface_create(dpy, win, DefaultVisual(dpy, 0), SIZEX, SIZEY);
	c=cairo_create(cs);

	initEngine();

	while(1) {
		while (XPending(dpy)) {
			XNextEvent(dpy, &e);
			if (e.type==Expose && e.xexpose.count<1) {
				voomDraw();
			}
			if (e.type==KeyPress || e.type==KeyRelease) {
				//Ignore autorepeat releases
				if (XEventsQueued(dpy, QueuedAfterReading)) {
					XPeekEvent(dpy, &ne);
					if (e.type==KeyRelease && ne.type == KeyPress && ne.xkey.time == e.xkey.time && ne.xkey.keycode == e.xkey.keycode) continue;
				}
				XKeyEvent k=e.xkey;
				if (k.keycode==9 && e.type==KeyPress) exit(0); //esc pressed
				if (k.keycode==111) { if (e.type==KeyPress) keys|=VOOM_KEY_UP; else keys&=~VOOM_KEY_UP; }
				if (k.keycode==116) { if (e.type==KeyPress) keys|=VOOM_KEY_DOWN; else keys&=~VOOM_KEY_DOWN; }
				if (k.keycode==113) { if (e.type==KeyPress) keys|=VOOM_KEY_LEFT; else keys&=~VOOM_KEY_LEFT; }
				if (k.keycode==114) { if (e.type==KeyPress) keys|=VOOM_KEY_RIGHT; else keys&=~VOOM_KEY_RIGHT; }
//				if (k.keycode==38 && e.type==KeyPress) myz+=3;
//				if (k.keycode==52 && e.type==KeyPress) myz-=3;
//				if (k.keycode==65 && e.type==KeyPress) mode++;
//				printf("%d\n", k.keycode);
			}
		}
		
		voomHandleKeys(keys);
		linesClear();
		voomDraw();
		linesDraw();
		usleep(40000);
	}
}

