; Copyright (C) 2015 Jeroen Domburg <jeroen at spritesmods.com>
;
; This library is free software: you can redistribute it and/or modify
; it under the terms of the GNU Lesser General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This library is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU Lesser General Public License for more details.
;
; You should have received a copy of the GNU Lesser General Public License
; along with this library.  If not, see <http://www.gnu.org/licenses/>.
	include "vectrex.i"

vecpos	EQU	$ca00
type	EQU $ca02
tmp		EQU	$ca04
rpcfn	EQU $cb00

;Cartridge header
	ORG 0
	fcb "g GCE 2015", $80		;'g' is copyright sign
	fdb music1					;music from the rom
	fcb $F8, $50, $20, -$56		; height, width, rel y, rel x (from 0,0)
	fcb "VOOM",$80; some game information ending with $80
	fcb 0		;				 end of game header

main
;copy rpc fn into place
	ldx #rpcfndat
	ldy #rpcfn
rpccopyloop
	lda ,x+
	sta ,y+
	cmpx #rpcfndatend
	bne rpccopyloop

	;set dp to $d0
	lda #$d0
	tfr a,dp
	setdp #$d0

restart
	jmp loadandgo

loop

	LDA #$20			;scale
	STA VIA_t1_cnt_lo
	LDX vecpos

	ldb #$ce 		;PUNZRO
	stb VIA_cntl ;aka PCNTRL
drawmore
	;We're at the start of a new vector.
	;Format: flags count y1 x1 [y2 x2 [...]]
	lda ,x+ ;Grab intensity
	cmpa #$ff ;No need to draw anything?
	beq drawdone
	bita #$40	;Need to zero integrators first?
	beq nozero

	;zero integrators: moves back to (0,0)
	ldb #$cc
	stb VIA_cntl  ;aka CNTRL
	clr VIA_shift_reg
	ldb #$03
	clr VIA_port_a
	stb VIA_port_b
	ldb #$02
	stb VIA_port_b
	stb VIA_port_b
	ldb #$1
	stb VIA_port_b
	anda #$3f

	ldb #$ce 		;PUNZRO
	stb VIA_cntl ;aka PCNTRL

nozero
	JSR Intensity_a

	LDA ,x+			;load scale
	STA VIA_t1_cnt_lo

	;Load and set Y coord
	ldb ,x+ ;x coord
	stb VIA_port_a
	clr VIA_port_b ;enable s/h strobe
	ldb ,x+ ;y coord
	nop
	inc VIA_port_b ;disable s/h
	stb VIA_port_a
	ldb #$FF
	stb VIA_shift_reg
	;Go draw; wait till done.
	clr VIA_t1_cnt_hi
	ldb #$40
drawwait
	bitb VIA_int_flags
	beq drawwait
	;We're done.
	clr VIA_shift_reg
	
	jmp drawmore

drawdone
	;We're done with this frame.
	ldb #$20
wrecloop
	;Check if vblank has ended by now
	bitb VIA_int_flags
	beq wrecloop

	ldd #$50c3 ;30fps
	std VIA_t2_lo
	jsr Recalibrate

loadandgo

	ldb #0
;Joystick handling
	jsr Joy_Digital
;X handling
	lda Vec_Joy_1_X
	beq xmovedone
	bpl xneg
	orb #4
	bra xmovedone
xneg
	orb #8
xmovedone

;Y handling
	lda Vec_Joy_1_Y
	beq ymovedone
	bpl yneg
	orb #2
	bra ymovedone
yneg
	orb #1
ymovedone
	stb $7ffe
	
	;Load next frame over rpc
	lda #66
	jsr rpcfn
	;Next frame starts at 0x800
	ldx #$800
	stx vecpos
	jmp loop

;Rpc function. Because this makes the cart unavailable, this
;will copied to SRAM. Call as rpcfn.
rpcfndat
	sta $7fff
rpcwaitloop
	lda $0
	cmpa #'g'
	bne rpcwaitloop
	lda $1
	cmpa #' '
	bne rpcwaitloop
	rts
rpcfndatend



	;zero integrators. WARNING: Moves shit back to (0,0)
;	ldb #$cc
;	stb VIA_cntl
;	clr VIA_shift_reg
